@@ -24,7 +24,6 @@ from neutron import opts as neutron_options
 from neutron.plugins.ml2 import plugin as ml2_plugin
 
 from networking_ovn.common import config as ovn_config
-from networking_ovn.common import ovn_client
 from networking_ovn.ml2 import mech_driver
 from networking_ovn import ovn_db_sync
 from networking_ovn.ovsdb import impl_idl_ovn
@@ -53,8 +52,6 @@ class OVNMechanismDriver(mech_driver.OVNMechanismDriver):
 
     @property
     def ovn_client(self):
-        if not self._ovn_client:
-            self._ovn_client = ovn_client.OVNClient(self._nb_ovn, self._sb_ovn)
         return self._ovn_client
 
     # Since we are not using the networking_ovn mechanism driver while syncing,
