@@ -116,7 +116,22 @@ class ChassisEvent(row_event.RowEvent):
         super(ChassisEvent, self).__init__(events, table, None)
         self.event_name = 'ChassisEvent'
 
+    def match_fn(self, event, row, old):
+        if event != self.ROW_UPDATE:
+            return True
+        # NOTE(lucasgomes): If the external_ids column wasn't updated
+        # (meaning, Chassis "gateway" status didn't change) just returns
+        if not hasattr(old, 'external_ids'):
+            return
+        if (old.external_ids.get('ovn-bridge-mappings') !=
+                row.external_ids.get('ovn-bridge-mappings')):
+            return True
+        f = utils.is_gateway_chassis
+        return f(old) != f(row)
+
     def run(self, event, row, old):
+        if not self.match_fn(event, row, old):
+            return
         host = row.hostname
         phy_nets = []
         if event != self.ROW_DELETE:
@@ -126,8 +141,33 @@ class ChassisEvent(row_event.RowEvent):
             phy_nets = list(mapping_dict)
 
         self.driver.update_segment_host_mapping(host, phy_nets)
+
         if utils.is_ovn_l3(self.l3_plugin):
-            self.l3_plugin.schedule_unhosted_gateways()
+            # If chassis lost physnet or has been
+            # deleted we can limit the scope and
+            # reschedule only ports from this chassis.
+            # In other cases we need to reschedule all gw ports.
+            kwargs = {'event_from_chassis': None}
+            if event == self.ROW_DELETE:
+                kwargs['event_from_chassis'] = row.name
+            elif event == self.ROW_UPDATE:
+                old_mappings = old.external_ids.get('ovn-bridge-mappings',
+                                                    set()) or set()
+                new_mappings = row.external_ids.get('ovn-bridge-mappings',
+                                                    set()) or set()
+                if old_mappings:
+                    old_mappings = set(old_mappings.split(','))
+                if new_mappings:
+                    new_mappings = set(new_mappings.split(','))
+
+                mappings_removed = old_mappings - new_mappings
+                mappings_added = new_mappings - old_mappings
+                if mappings_removed and not mappings_added:
+                    # Mapping has been only removed. So we can
+                    # limit scope of rescheduling only to impacted
+                    # gateway chassis.
+                    kwargs['event_from_chassis'] = row.name
+            self.l3_plugin.schedule_unhosted_gateways(**kwargs)
 
 
 class PortBindingChassisUpdateEvent(BaseEvent):
