@@ -76,6 +76,7 @@ class FakeOvsdbNbOvnIdl(object):
         self.delete_address_set = mock.Mock()
         self.update_address_set = mock.Mock()
         self.get_all_chassis_gateway_bindings = mock.Mock()
+        self.get_chassis_gateways = mock.Mock()
         self.get_gateway_chassis_binding = mock.Mock()
         self.get_unhosted_gateways = mock.Mock()
         self.add_dhcp_options = mock.Mock()
@@ -138,9 +139,13 @@ class FakeOvsdbNbOvnIdl(object):
         self.db_list_rows = mock.Mock()
         self.lsp_list = mock.MagicMock()
         self.db_find = mock.Mock()
+        self.db_find_rows = mock.Mock()
         self.db_set = mock.Mock()
         self.db_clear = mock.Mock()
         self.db_remove = mock.Mock()
+        self.set_lswitch_port_to_virtual_type = mock.Mock()
+        self.unset_lswitch_port_to_virtual_type = mock.Mock()
+        self.ls_get = mock.Mock()
 
 
 class FakeOvsdbSbOvnIdl(object):
@@ -159,6 +164,8 @@ class FakeOvsdbSbOvnIdl(object):
             ('fake', 'fake-dp')
         self.get_chassis_and_physnets = mock.Mock()
         self.get_gateway_chassis_from_cms_options = mock.Mock()
+        self.is_col_present = mock.Mock()
+        self.is_col_present.return_value = False
 
 
 class FakeOvsdbTransaction(object):
