@@ -16,7 +16,9 @@ import mock
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
+from neutron_lib.callbacks import resources
 from neutron_lib import exceptions as n_exc
+from neutron_lib.services.trunk import constants as trunk_consts
 
 from networking_ovn.common.constants import OVN_ML2_MECH_DRIVER_NAME
 from networking_ovn.ml2 import trunk_driver
@@ -24,7 +26,7 @@ from networking_ovn.tests.unit import fakes
 
 from neutron.objects.ports import Port
 from neutron.objects.ports import PortBinding
-from neutron.services.trunk import constants as trunk_consts
+
 from neutron.tests import base
 
 from oslo_config import cfg
@@ -78,6 +80,8 @@ class TestTrunkHandler(base.BaseTestCase):
         self.mock_get_port.side_effect = lambda ctxt, id: (
             self.sub_port_1_obj if id == 'sub_port_1' else (
                 self.sub_port_2_obj if id == 'sub_port_2' else None))
+        self.mock_port_update = mock.patch(
+            "neutron.objects.ports.Port.update").start()
         self.mock_update_pb = mock.patch(
             "neutron.objects.ports.PortBinding.update_object").start()
         self.mock_clear_levels = mock.patch(
@@ -88,7 +92,7 @@ class TestTrunkHandler(base.BaseTestCase):
             mock_uuid.return_value = port_id
             port = Port()
             port.id = port_id
-            port.binding = PortBinding(profile={}, host='foo.com')
+            port.bindings = [PortBinding(profile={}, host='foo.com')]
         return port
 
     def _assert_calls(self, mock, expected_calls):
@@ -107,10 +111,14 @@ class TestTrunkHandler(base.BaseTestCase):
         self.trunk_1.sub_ports = [self.sub_port_1, self.sub_port_2]
         self.handler.trunk_created(self.trunk_1)
 
+        calls = [mock.call(), mock.call()]
+        self._assert_calls(self.mock_port_update, calls)
+
         calls = [
             mock.call(mock.ANY,
                       {'profile': {'parent_name': trunk.port_id,
-                                   'tag': s_port.segmentation_id}},
+                                   'tag': s_port.segmentation_id},
+                       'vif_type': portbindings.VIF_TYPE_OVS},
                       host=mock.ANY,
                       port_id=s_port.port_id)
             for trunk, s_port in [(self.trunk_1, self.sub_port_1),
@@ -137,8 +145,10 @@ class TestTrunkHandler(base.BaseTestCase):
         self.handler.trunk_created(self.trunk_1)
         self.mock_update_pb.assert_called_once_with(
             mock.ANY, {'profile': {'parent_name': self.sub_port_1.trunk_id,
-                                   'tag': self.sub_port_1.segmentation_id}},
+                                   'tag': self.sub_port_1.segmentation_id},
+                       'vif_type': portbindings.VIF_TYPE_OVS},
             host='foo.com', port_id=self.sub_port_1.port_id)
+        self.mock_port_update.assert_not_called()
         self.plugin_driver._nb_ovn.set_lswitch_port.assert_not_called()
 
     def test_delete_trunk(self):
@@ -149,23 +159,24 @@ class TestTrunkHandler(base.BaseTestCase):
         self.mock_clear_levels.assert_not_called()
 
         self.trunk_1.sub_ports = [self.sub_port_1, self.sub_port_2]
-        self.sub_port_1_obj.binding.profile.update({
+        self.sub_port_1_obj.bindings[0].profile.update({
             'tag': self.sub_port_1.segmentation_id,
             'parent_name': self.sub_port_1.trunk_id,
             'foo_field': self.sub_port_1.trunk_id})
-        self.sub_port_2_obj.binding.profile.update({
+        self.sub_port_2_obj.bindings[0].profile.update({
             'tag': self.sub_port_2.segmentation_id,
             'parent_name': self.sub_port_2.trunk_id,
             'foo_field': self.sub_port_2.trunk_id})
         self.handler.trunk_deleted(self.trunk_1)
 
+        calls = [mock.call(), mock.call()]
+        self._assert_calls(self.mock_port_update, calls)
+
         calls = [
             mock.call(
                 mock.ANY,
                 {'profile': {'foo_field': s_port.trunk_id},
-                 'host': '',
-                 'vif_type': portbindings.VIF_TYPE_UNBOUND,
-                 'vif_details': ''},
+                 'vif_type': portbindings.VIF_TYPE_UNBOUND},
                 host='foo.com',
                 port_id=s_port.port_id)
             for trunk, s_port in [(self.trunk_1, self.sub_port_1),
@@ -189,16 +200,14 @@ class TestTrunkHandler(base.BaseTestCase):
         self._assert_calls(self.plugin_driver._nb_ovn.set_lswitch_port, calls)
 
     def test_delete_trunk_key_not_found(self):
-        self.sub_port_1_obj.binding.profile.update({
+        self.sub_port_1_obj.bindings[0].profile.update({
             'foo_field': self.sub_port_1.trunk_id})
         self.trunk_1.sub_ports = [self.sub_port_1]
         self.handler.trunk_deleted(self.trunk_1)
         calls = [
             mock.call(mock.ANY,
                       {'profile': {'foo_field': s_port.trunk_id},
-                       'host': '',
-                       'vif_type': portbindings.VIF_TYPE_UNBOUND,
-                       'vif_details': ''},
+                       'vif_type': portbindings.VIF_TYPE_UNBOUND},
                       host='foo.com',
                       port_id=s_port.port_id)
             for trunk, s_port in [(self.trunk_1, self.sub_port_1)]]
@@ -231,9 +240,9 @@ class TestTrunkHandler(base.BaseTestCase):
         self.handler.trunk_deleted(self.trunk_1)
         self.mock_update_pb.assert_called_once_with(
             mock.ANY, {'profile': {},
-                       'vif_type': portbindings.VIF_TYPE_UNBOUND,
-                       'host': '', 'vif_details': ''},
+                       'vif_type': portbindings.VIF_TYPE_UNBOUND},
             host='foo.com', port_id=self.sub_port_1.port_id)
+        self.mock_port_update.assert_not_called()
         self.plugin_driver._nb_ovn.set_lswitch_port.assert_not_called()
         self.mock_clear_levels.assert_not_called()
 
@@ -244,7 +253,7 @@ class TestTrunkHandler(base.BaseTestCase):
         set_s.assert_called_once_with(
             self.trunk_1.port_id, [self.sub_port_1, self.sub_port_2])
         self.trunk_1.update.assert_called_once_with(
-            status=trunk_consts.ACTIVE_STATUS)
+            status=trunk_consts.TRUNK_ACTIVE_STATUS)
 
     def test_subports_deleted(self):
         with mock.patch.object(self.handler, '_unset_sub_ports') as unset_s:
@@ -253,7 +262,7 @@ class TestTrunkHandler(base.BaseTestCase):
         unset_s.assert_called_once_with(
             [self.sub_port_1, self.sub_port_2])
         self.trunk_1.update.assert_called_once_with(
-            status=trunk_consts.ACTIVE_STATUS)
+            status=trunk_consts.TRUNK_ACTIVE_STATUS)
 
     def _fake_trunk_event_payload(self):
         payload = mock.Mock()
@@ -282,7 +291,7 @@ class TestTrunkHandler(base.BaseTestCase):
             fake_payload.current_trunk.port_id,
             fake_payload.current_trunk.sub_ports)
         fake_payload.current_trunk.update.assert_called_once_with(
-            status=trunk_consts.ACTIVE_STATUS)
+            status=trunk_consts.TRUNK_ACTIVE_STATUS)
 
     @mock.patch.object(trunk_driver.OVNTrunkHandler, '_unset_sub_ports')
     def test_trunk_event_delete(self, unset_subports):
@@ -362,15 +371,15 @@ class TestTrunkDriver(base.BaseTestCase):
         with mock.patch.object(registry, 'subscribe') as mock_subscribe:
             driver.register(mock.ANY, mock.ANY, mock.Mock())
             calls = [mock.call.mock_subscribe(mock.ANY,
-                                              trunk_consts.TRUNK,
+                                              resources.TRUNK,
                                               events.AFTER_CREATE),
                      mock.call.mock_subscribe(mock.ANY,
-                                              trunk_consts.SUBPORTS,
+                                              resources.SUBPORTS,
                                               events.AFTER_CREATE),
                      mock.call.mock_subscribe(mock.ANY,
-                                              trunk_consts.TRUNK,
+                                              resources.TRUNK,
                                               events.AFTER_DELETE),
                      mock.call.mock_subscribe(mock.ANY,
-                                              trunk_consts.SUBPORTS,
+                                              resources.SUBPORTS,
                                               events.AFTER_DELETE)]
             mock_subscribe.assert_has_calls(calls, any_order=True)
